@@ -28,7 +28,6 @@ Approach:
 """
 
 from urllib.error import (URLError, HTTPError)
-from urllib.parse import urlparse
 from urllib.request import urlopen
 import codecs
 import functools
@@ -37,6 +36,7 @@ import os.path
 import re
 import subprocess
 import sys
+import tempfile
 
 from abc import (ABC, abstractmethod)
 
@@ -52,7 +52,7 @@ from ubuntutools.lp.udtexceptions import (PackageNotFoundException,
                                           SeriesNotFoundException,
                                           PocketDoesNotExistError,
                                           InvalidDistroValueError)
-from ubuntutools.misc import (download, verify_file_checksum, verify_file_checksums)
+from ubuntutools.misc import (download, download_text, verify_file_checksum, verify_file_checksums)
 from ubuntutools.version import Version
 
 import logging
@@ -180,15 +180,10 @@ class SourcePackage(ABC):
 
         # If provided a dscfile, process it now to set our source and version
         if self._dsc_source:
-            filename = os.path.basename(urlparse(self._dsc_source).path)
-            dst = os.path.join(self.workdir, filename)
-            download(self._dsc_source, dst)
-            with open(dst, 'rb') as f:
-                self._dsc = Dsc(f.read())
+            self._dsc = Dsc(download_text(self._dsc_source, mode='rb'))
             self.source = self._dsc['Source']
             self._version = Version(self._dsc['Version'])
-            if self._verify_signature:
-                self._check_dsc_signature()
+            self._check_dsc_signature()
 
     @property
     def lp_spph(self):
@@ -283,10 +278,14 @@ class SourcePackage(ABC):
     def dsc(self):
         "Return the Dsc"
         if not self._dsc:
-            self.pull_dsc()
-            with open(self.dsc_pathname, 'rb') as f:
-                self._dsc = Dsc(f.read())
-            if self._verify_signature:
+            if self._dsc_source:
+                raise RuntimeError('Internal error: we have a dsc file but dsc not set')
+            urls = self._source_urls(self.dsc_name)
+            with tempfile.TemporaryDirectory() as d:
+                tmpdsc = os.path.join(d, self.dsc_name)
+                self._download_file_from_urls(urls, tmpdsc)
+                with open(tmpdsc, 'rb') as f:
+                    self._dsc = Dsc(f.read())
                 self._check_dsc_signature()
         return self._dsc
 
@@ -336,6 +335,8 @@ class SourcePackage(ABC):
 
     def _check_dsc_signature(self):
         "Check that the dsc signature matches what we are expecting"
+        if not self._verify_signature:
+            return
         try:
             gpg_info = self.dsc.get_gpg_info((
                 '/usr/share/keyrings/debian-keyring.gpg',
@@ -375,8 +376,11 @@ class SourcePackage(ABC):
         return True
 
     def _download_file(self, url, filename, size=0, dscverify=False, sha1sum=None, sha256sum=None):
-        "Download url to filename in workdir."
-        pathname = os.path.join(self.workdir, filename)
+        "Download url to filename; will be put in workdir unless filename is absolute path."
+        if os.path.isabs(filename):
+            pathname = filename
+        else:
+            pathname = os.path.join(self.workdir, filename)
 
         if self._verify_file(pathname, dscverify, sha1sum, sha256sum, size):
             Logger.info('Using existing file %s', filename)
@@ -405,12 +409,23 @@ class SourcePackage(ABC):
         raise DownloadError('Failed to download %s' % filename)
 
     def pull_dsc(self):
-        "Pull dscfile into workdir"
-        urls = self._source_urls(self.dsc_name)
-        self._download_file_from_urls(urls, self.dsc_name)
+        '''DEPRECATED
+
+        This method is badly named and historically has only 'pulled' the
+        dsc into memory, not actually to a file. Since the other 'pull' methods
+        actually 'pull' to real files, this method makes no sense; additionally
+        there is no need for this method since we can 'pull' the dsc into
+        memory when the .dsc property is accessed.
+
+        This method no longer does anything at all and should not be used by
+        anyone.
+        '''
+        Logger.debug('Please fix your program: the "pull_dsc" method is deprecated')
 
     def pull(self):
         "Pull into workdir"
+        with open(self.dsc_pathname, 'wb') as f:
+            f.write(self.dsc.raw_text)
         for entry in self.dsc['Files']:
             name = entry['name']
             urls = self._source_urls(name)
