@@ -21,10 +21,6 @@
 #
 #   Based on code written by Jonathan Davies <jpds@ubuntu.com>
 
-# Uncomment for tracing LP API calls
-# import httplib2
-# httplib2.debuglevel = 1
-
 import collections
 import os
 import re
@@ -32,15 +28,16 @@ import re
 from copy import copy
 
 from debian.changelog import Changelog
-from httplib2 import Http, HttpLib2Error
 from launchpadlib.launchpad import Launchpad as LP
 from launchpadlib.errors import HTTPError
 from lazr.restfulclient.resource import Entry
+from urllib.error import URLError
 from urllib.parse import urlparse
 
 from ubuntutools.version import Version
 from ubuntutools.lp import (service, api_version)
-from ubuntutools.misc import (host_architecture,
+from ubuntutools.misc import (download_text,
+                              host_architecture,
                               DEFAULT_POCKETS, POCKETS,
                               DEFAULT_STATUSES, STATUSES)
 from ubuntutools.lp.udtexceptions import (AlreadyLoggedInError,
@@ -65,6 +62,8 @@ __all__ = [
     'Launchpad',
     'PackageUpload',
     'PersonTeam',
+    'Project',
+    'ProjectSeries',
     'SourcePackagePublishingHistory',
     ]
 
@@ -106,7 +105,7 @@ class _Launchpad(object):
 
     def __getattr__(self, attr):
         if not self.logged_in:
-            self.login()
+            self.login_anonymously()
         return getattr(self.__lp, attr)
 
     def __call__(self):
@@ -850,14 +849,10 @@ class SourcePackagePublishingHistory(BaseWrapper):
                 return None
 
             try:
-                response, changelog = Http().request(url)
-            except HttpLib2Error as e:
-                Logger.error(str(e))
+                self._changelog = download_text(url)
+            except URLError as e:
+                Logger.error(f"Exception while downloading '{url}': {e}")
                 return None
-            if response.status != 200:
-                Logger.error('%s: %s %s' % (url, response.status, response.reason))
-                return None
-            self._changelog = changelog
 
         if since_version is None:
             return self._changelog
@@ -886,6 +881,8 @@ class SourcePackagePublishingHistory(BaseWrapper):
           size: the size of the source file
         Also, this function adds a 'filename' field:
           filename: the filename parsed from the url path
+        Note that while all the keys will be in the dict, their values
+        may be None.
         '''
         if not self._source_urls:
             urls = self._lpobject.sourceFileUrls(include_meta=True)
@@ -893,6 +890,10 @@ class SourcePackagePublishingHistory(BaseWrapper):
                 Logger.warning('SPPH %s_%s has no sourceFileUrls' %
                                (self.getPackageName(), self.getVersion()))
             for u in urls:
+                # make sure mandatory fields are present
+                for field in ['url', 'sha1', 'sha256', 'size']:
+                    if field not in u:
+                        u[field] = None
                 u['filename'] = os.path.basename(urlparse(u['url']).path)
             self._source_urls = urls
 
@@ -1141,6 +1142,8 @@ class BinaryPackagePublishingHistory(BaseWrapper):
           size: the size of the binary file
         Also, this function adds a 'filename' field:
           filename: the filename parsed from the url path
+        Note that while all the keys will be in the dict, their values
+        may be None.
         '''
         if not self._binary_urls:
             try:
@@ -1152,6 +1155,10 @@ class BinaryPackagePublishingHistory(BaseWrapper):
                 Logger.warning('BPPH %s_%s has no binaryFileUrls' %
                                (self.getPackageName(), self.getVersion()))
             for u in urls:
+                # make sure mandatory fields are present
+                for field in ['url', 'sha1', 'sha256', 'size']:
+                    if field not in u:
+                        u[field] = None
                 u['filename'] = os.path.basename(urlparse(u['url']).path)
             self._binary_urls = urls
 
@@ -1395,6 +1402,44 @@ class PersonTeam(BaseWrapper, metaclass=MetaPersonTeam):
         return Archive(self._lpobject.getPPAByName(name=name))
 
 
+class Project(BaseWrapper):
+    '''
+    Wrapper class around a LP project object.
+    '''
+    resource_type = ('project')
+
+    def __init__(self, *args):
+        self._series = None
+
+    @property
+    def series(self):
+        """Get a list of all ProjectSeries
+
+        The list will be sorted by date_created, in descending order.
+        """
+        if not self._series:
+            series = [ProjectSeries(s['self_link']) for s in
+                      Launchpad.load(self._lpobject.series_collection_link).entries]
+            self._series = sorted(series, key=lambda s: s.date_created, reverse=True)
+        return self._series.copy()
+
+    @classmethod
+    def fetch(cls, project):
+        '''
+        Fetch the project object identified by 'project' from LP.
+        '''
+        if not isinstance(project, str):
+            raise TypeError("Don't know what do with '%r'" % project)
+        return Project(Launchpad.projects(project))
+
+
+class ProjectSeries(BaseWrapper):
+    '''
+    Wrapper class around a LP project_series object.
+    '''
+    resource_type = ('project_series')
+
+
 class Build(BaseWrapper):
     '''
     Wrapper class around a build object.
