@@ -38,7 +38,8 @@ from ubuntutools.archive import (UbuntuSourcePackage, DebianSourcePackage,
                                  PersonalPackageArchiveSourcePackage)
 from ubuntutools.config import UDTConfig
 from ubuntutools.lp.lpapicache import (Distribution, Launchpad)
-from ubuntutools.lp.udtexceptions import (SeriesNotFoundException,
+from ubuntutools.lp.udtexceptions import (AlreadyLoggedInError,
+                                          SeriesNotFoundException,
                                           PackageNotFoundException,
                                           PocketDoesNotExistError,
                                           InvalidDistroValueError)
@@ -48,8 +49,10 @@ from ubuntutools.misc import (split_release_pocket,
                               UPLOAD_QUEUE_STATUSES,
                               STATUSES)
 
-from ubuntutools import _loggingBasicConfig
 
+# by default we use standard logging.getLogger() and only use
+# ubuntutools.getLogger() in PullPkg().main()
+from ubuntutools import getLogger as ubuntutools_getLogger
 import logging
 Logger = logging.getLogger(__name__)
 
@@ -97,7 +100,7 @@ class PullPkg(object):
         unexpected errors will flow up to the caller.
         On success, this simply returns.
         """
-        _loggingBasicConfig()
+        Logger = ubuntutools_getLogger()
 
         try:
             cls(*args, **kwargs).pull()
@@ -105,7 +108,8 @@ class PullPkg(object):
         except KeyboardInterrupt:
             Logger.info('User abort.')
         except (PackageNotFoundException, SeriesNotFoundException,
-                PocketDoesNotExistError, InvalidDistroValueError) as e:
+                PocketDoesNotExistError, InvalidDistroValueError,
+                InvalidPullValueError) as e:
             Logger.error(str(e))
             sys.exit(errno.ENOENT)
 
@@ -129,6 +133,8 @@ class PullPkg(object):
         # use add_help=False because we do parse_known_args() below, and if
         # that sees --help then it exits immediately
         parser = ArgumentParser(add_help=False)
+        parser.add_argument('-L', '--login', action='store_true',
+                            help="Login to Launchpad")
         parser.add_argument('-v', '--verbose', action='count', default=0,
                             help="Increase verbosity/debug")
         parser.add_argument('-d', '--download-only', action='store_true',
@@ -226,15 +232,7 @@ class PullPkg(object):
 
     def parse_release(self, distro, release):
         if distro == DISTRO_UCA:
-            # UCA is special; it is specified UBUNTURELEASE-UCARELEASE or just
-            # UCARELEASE.  The user could also specify UCARELEASE-POCKET.  But UCA
-            # archives always correspond to only one UBUNTURELEASE, and UCA archives
-            # have only the Release pocket, so only UCARELEASE matters to us.
-            for r in release.split('-'):
-                if UbuntuCloudArchiveSourcePackage.isValidRelease(r):
-                    Logger.debug("Using UCA release '%s'", r)
-                    return (r, None)
-            raise SeriesNotFoundException('UCA release {} not found.'.format(release))
+            return UbuntuCloudArchiveSourcePackage.parseReleaseAndPocket(release)
 
         # Check if release[-pocket] is specified
         (release, pocket) = split_release_pocket(release, default=None)
@@ -343,7 +341,7 @@ class PullPkg(object):
 
         mirrors = []
         if options['mirror']:
-            mirrors.append(options['mirror'])
+            mirrors.extend(options['mirror'])
         if pull == PULL_DDEBS:
             config = UDTConfig(options['no_conf'])
             ddebs_mirror = config.get_value(distro.upper() + '_DDEBS_MIRROR')
@@ -386,12 +384,18 @@ class PullPkg(object):
 
         Logger.debug("pullpkg options: %s", options)
 
-        # Login anonymously to LP
-        Launchpad.login_anonymously()
-
         pull = options['pull']
         distro = options['distro']
 
+        if options['login']:
+            Logger.debug("Logging in to Launchpad:")
+            try:
+                Launchpad.login()
+            except AlreadyLoggedInError:
+                Logger.error("Launchpad singleton has already performed a login, "
+                             "and its design prevents another login")
+                Logger.warning("Continuing anyway, with existing Launchpad instance")
+
         params = self._get_params(options)
         package = params['package']
 
@@ -556,7 +560,7 @@ class PullPkg(object):
             if not urls:
                 Logger.error("No source files to download")
             for url in urls:
-                download(url)
+                download(url, os.getcwd())
         else:
             name = '.*'
             if pull == PULL_DEBS:
@@ -574,4 +578,4 @@ class PullPkg(object):
             for url in urls:
                 filename = os.path.basename(urlparse(url).path)
                 if re.match(name, filename):
-                    download(url)
+                    download(url, os.getcwd())
