@@ -16,13 +16,14 @@
 # PERFORMANCE OF THIS SOFTWARE.
 
 import locale
-import mock
 import os
 # import sys
+import unittest
+
 from io import StringIO
+from unittest import mock
 
 from ubuntutools.config import UDTConfig, ubu_email
-from ubuntutools.test import unittest
 
 
 class ConfigTestCase(unittest.TestCase):
